@@ -16,42 +16,70 @@ use std::sync::Mutex;
 
 use crate::enum_dispatch_item;
 
+/// Uniquely identifies a trait or an enum. This is based on its name and number of arguments.
+#[derive(PartialEq, Eq, Hash, Clone)]
+struct UniqueItemId {
+    item_name: String,
+    num_generics: usize,
+}
+
+impl UniqueItemId {
+    /// Convenience constructor for UniqueItemId.
+    pub fn new(item_name: String, num_generics: usize) -> Self {
+        Self {
+            item_name,
+            num_generics,
+        }
+    }
+}
+
 // Magical storage for trait definitions so that they can be used when parsing other syntax
 // structures.
-static TRAIT_DEFS: Lazy<Mutex<HashMap<String, String>>> = Lazy::new(|| Mutex::new(HashMap::new()));
-static ENUM_DEFS: Lazy<Mutex<HashMap<String, String>>> = Lazy::new(|| Mutex::new(HashMap::new()));
-static DEFERRED_LINKS: Lazy<Mutex<HashMap<String, Vec<String>>>> =
+static TRAIT_DEFS: Lazy<Mutex<HashMap<UniqueItemId, String>>> =
     Lazy::new(|| Mutex::new(HashMap::new()));
-static ENUM_CONVERSION_IMPLS_DEFS: Lazy<Mutex<HashSet<String>>> =
+static ENUM_DEFS: Lazy<Mutex<HashMap<UniqueItemId, String>>> =
+    Lazy::new(|| Mutex::new(HashMap::new()));
+static DEFERRED_LINKS: Lazy<Mutex<HashMap<UniqueItemId, Vec<UniqueItemId>>>> =
+    Lazy::new(|| Mutex::new(HashMap::new()));
+static ENUM_CONVERSION_IMPLS_DEFS: Lazy<Mutex<HashSet<UniqueItemId>>> =
     Lazy::new(|| Mutex::new(HashSet::new()));
 
 /// Store a trait definition for future reference.
 pub fn cache_trait(item: syn::ItemTrait) {
-    let identname = item.ident.to_string();
+    let uid = UniqueItemId::new(item.ident.to_string(), item.generics.type_params().count());
     TRAIT_DEFS
         .lock()
         .unwrap()
-        .insert(identname, item.into_token_stream().to_string());
+        .insert(uid, item.into_token_stream().to_string());
 }
 
 /// Store an enum definition for future reference.
 pub fn cache_enum_dispatch(item: enum_dispatch_item::EnumDispatchItem) {
-    let identname = item.ident.to_string();
+    let uid = UniqueItemId::new(item.ident.to_string(), item.generics.type_params().count());
     ENUM_DEFS
         .lock()
         .unwrap()
-        .insert(identname, item.into_token_stream().to_string());
+        .insert(uid, item.into_token_stream().to_string());
 }
 
 /// Store whether a From/TryInto definition has been defined once for an enum.
-pub fn cache_enum_conversion_impls_defined(item: syn::Ident) {
-    let identname = item.to_string();
-    ENUM_CONVERSION_IMPLS_DEFS.lock().unwrap().insert(identname);
+pub fn cache_enum_conversion_impls_defined(item: syn::Ident, num_generics: usize) {
+    let uid = UniqueItemId::new(item.to_string(), num_generics);
+    ENUM_CONVERSION_IMPLS_DEFS.lock().unwrap().insert(uid);
 }
 
 /// Cache a "link" to be fulfilled once the needed definition is also cached.
-pub fn defer_link(needed: &::proc_macro2::Ident, cached: &::proc_macro2::Ident) {
-    let (needed, cached) = (needed.to_string(), cached.to_string());
+///
+/// The number of generic arguments is also cached and must be equal in order to fulfill a link,
+/// however the actual generic arguments themselves may have different names.
+pub fn defer_link(
+    (needed, needed_num_generics): (&::proc_macro2::Ident, usize),
+    (cached, cached_num_generics): (&::proc_macro2::Ident, usize),
+) {
+    let (needed, cached) = (
+        UniqueItemId::new(needed.to_string(), needed_num_generics),
+        UniqueItemId::new(cached.to_string(), cached_num_generics),
+    );
     let mut deferred_links = DEFERRED_LINKS.lock().unwrap();
     if deferred_links.contains_key(&needed) {
         deferred_links
@@ -70,11 +98,14 @@ pub fn defer_link(needed: &::proc_macro2::Ident, cached: &::proc_macro2::Ident)
 
 /// Returns a list of all of the trait definitions that were previously linked to the supplied enum
 /// name.
-pub fn fulfilled_by_enum(defname: &::proc_macro2::Ident) -> Vec<syn::ItemTrait> {
+pub fn fulfilled_by_enum(
+    defname: &::proc_macro2::Ident,
+    num_generic_args: usize,
+) -> Vec<syn::ItemTrait> {
     let idents = match DEFERRED_LINKS
         .lock()
         .unwrap()
-        .remove_entry(&defname.to_string())
+        .remove_entry(&UniqueItemId::new(defname.to_string(), num_generic_args))
     {
         Some((_, links)) => links,
         None => vec![],
@@ -94,11 +125,12 @@ pub fn fulfilled_by_enum(defname: &::proc_macro2::Ident) -> Vec<syn::ItemTrait>
 /// name.
 pub fn fulfilled_by_trait(
     defname: &::proc_macro2::Ident,
+    num_generic_args: usize,
 ) -> Vec<enum_dispatch_item::EnumDispatchItem> {
     let idents = match DEFERRED_LINKS
         .lock()
         .unwrap()
-        .remove_entry(&defname.to_string())
+        .remove_entry(&UniqueItemId::new(defname.to_string(), num_generic_args))
     {
         Some((_, links)) => links,
         None => vec![],
@@ -115,9 +147,9 @@ pub fn fulfilled_by_trait(
 }
 
 /// Returns true if From/TryInto was already defined for this enum
-pub fn conversion_impls_def_by_enum(item: &syn::Ident) -> bool {
+pub fn conversion_impls_def_by_enum(item: &syn::Ident, num_generics: usize) -> bool {
     ENUM_CONVERSION_IMPLS_DEFS
         .lock()
         .unwrap()
-        .contains(&item.to_string())
+        .contains(&UniqueItemId::new(item.to_string(), num_generics))
 }
