@@ -326,12 +326,12 @@ use quote::{ToTokens, TokenStreamExt};
 mod attributed_parser;
 /// Provides local storage for enum and trait definitions so that they can be accessed later.
 mod cache;
+/// Provides a custom syntax specification for the arguments to an `#[enum_dispatch(...)]` attribute.
+mod enum_dispatch_arg_list;
 /// Provides a custom syntax specification for enum dispatch syntax blocks.
 mod enum_dispatch_item;
 /// Provides a custom syntax specification for the variants of enum dispatch syntax blocks.
 mod enum_dispatch_variant;
-/// Provides a custom syntax specification for the arguments to an `#[enum_dispatch(...)]` attribute.
-mod enum_dispatch_arg_list;
 /// Provides utilities for building enum dispatch implementations.
 mod expansion;
 /// Convenience trait for token parsing.
@@ -380,14 +380,37 @@ pub fn enum_dispatch(attr: TokenStream, item: TokenStream) -> TokenStream {
             .arg_list
             .into_iter()
             .for_each(|p| {
-                let attr_name = p.get_ident()
-                    .expect("Paths and generics in `#[enum_dispatch(...)]` are not supported.");
+                if p.leading_colon.is_some() || p.segments.len() != 1 {
+                    panic!("Paths in `#[enum_dispatch(...)]` are not supported.");
+                }
+                let syn::PathSegment {
+                    ident: attr_name,
+                    arguments: attr_generics
+                } = p.segments.last().unwrap();
+                let attr_generics = match attr_generics.clone() {
+                    syn::PathArguments::None => vec![],
+                    syn::PathArguments::AngleBracketed(args) => {
+                        assert!(args.colon2_token.is_none());
+                        args.args.iter().map(|generic_arg| {
+                            match generic_arg {
+                                syn::GenericArgument::Type(syn::Type::Path(t)) if t.qself.is_none() => Some(t.path.get_ident().expect("Generic binding paths in #[enum_dispatch(...)] are not supported").clone()),
+                                syn::GenericArgument::Type(syn::Type::Infer(_)) => None,
+                                syn::GenericArgument::Type(_) => panic!("Generics in #[enum_dispatch(...)] must be identifiers"),
+                                syn::GenericArgument::Lifetime(_) => panic!("Lifetime generics in #[enum_dispatch(...)] are not supported"),
+                                syn::GenericArgument::Binding(_) => panic!("Generic equality constraints in #[enum_dispatch(...)] are not supported"),
+                                syn::GenericArgument::Constraint(_) => panic!("Generic trait constraints in #[enum_dispatch(...)] are not supported"),
+                                syn::GenericArgument::Const(_) => panic!("Const expression generics in #[enum_dispatch(...)] are not supported"),
+                            }
+                        }).collect::<Vec<_>>()
+                    }
+                    syn::PathArguments::Parenthesized(_) => panic!("Expected angle bracketed generic arguments, found parenthesized arguments"),
+                };
                 match &new_block {
                     attributed_parser::ParsedItem::Trait(traitdef) => {
-                        cache::defer_link(attr_name, &traitdef.ident)
+                        cache::defer_link((attr_name, attr_generics.len()), (&traitdef.ident, traitdef.generics.type_params().count()))
                     }
                     attributed_parser::ParsedItem::EnumDispatch(enumdef) => {
-                        cache::defer_link(attr_name, &enumdef.ident)
+                        cache::defer_link((attr_name, attr_generics.len()), (&enumdef.ident, enumdef.generics.type_params().count()))
                     }
                 }
             });
@@ -397,13 +420,15 @@ pub fn enum_dispatch(attr: TokenStream, item: TokenStream) -> TokenStream {
     // *all* of the span information from being lost.
     match new_block {
         attributed_parser::ParsedItem::Trait(traitdef) => {
-            let additional_enums = cache::fulfilled_by_trait(&traitdef.ident);
+            let additional_enums =
+                cache::fulfilled_by_trait(&traitdef.ident, traitdef.generics.type_params().count());
             for enumdef in additional_enums {
                 expanded.append_all(add_enum_impls(enumdef, traitdef.clone()));
             }
         }
         attributed_parser::ParsedItem::EnumDispatch(enumdef) => {
-            let additional_traits = cache::fulfilled_by_enum(&enumdef.ident);
+            let additional_traits =
+                cache::fulfilled_by_enum(&enumdef.ident, enumdef.generics.type_params().count());
             for traitdef in additional_traits {
                 expanded.append_all(add_enum_impls(enumdef.clone(), traitdef));
             }
